@@ -6,21 +6,45 @@ use Error qw(:try);
 
 use base qw(XAO::testcases::FS::base);
 
-##
-# MySQL is noisy about mistakes that we expect. So we hide DBD
-# messages.
-#
-use vars qw(*SE);
-sub stderr_stop {
-    open(SE,">&STDERR");
-    open(STDERR,">/dev/null");
-}
-sub stderr_restore {
-    open(STDERR,">&SE");
-    close(SE);
+sub test_utf8_non_bmp {
+    my $self=shift;
+
+    my $odb=$self->get_odb();
+
+    my $global=$odb->fetch('/');
+    $self->assert(ref($global), "Failure getting / reference");
+
+    $global->add_placeholder(
+        name        => 'text',
+        type        => 'text',
+        charset     => 'utf8',
+        maxlength   => 50,
+    );
+
+    use utf8;
+
+    # The non-BMP (Basic Multilingual Plane, 0x0000-0xffff) unicode
+    # characters don't work with MySQL. They result in the string
+    # truncation at the unicode character.
+    #
+    my $unicode="Smile - \x{1f600} - After";
+    my $result='';
+    my $error;
+    try {
+        $global->put(text => $unicode);
+        $result=$global->get('text');
+    }
+    otherwise {
+        my $etext=''.shift;
+        dprint "Expected error: $etext";
+        $error=1;
+    };
+
+    $self->assert($error,
+        "Expected a failure for supplemental unicode string '$unicode', got '$result'");
 }
 
-sub test_update_field {
+sub test_space_stripping {
     my $self=shift;
 
     my $odb=$self->get_odb();
@@ -32,10 +56,7 @@ sub test_update_field {
     # Spaces at the end of string are chopped off at least by
     # MySQL. Documented bug.
     #
-    foreach my $text (q('"~!@#$%^&*_+=[]{}),
-                      '()',
-                      '  Spaces  .' ,
-                      'Test Project') {
+    foreach my $text ("   aaa   .") {
         $global->put(project => $text);
         my $got=$global->get('project');
         $self->assert($got eq $text,
@@ -43,6 +64,46 @@ sub test_update_field {
     }
 }
 
+sub test_8bit_transparency {
+    my $self=shift;
+
+    my $odb=$self->get_odb();
+
+    my $global=$odb->fetch('/');
+    $self->assert(ref($global), "Failure getting / reference");
+
+    ##
+    # For compatibility no charset means binary transparency, checking for it
+    #
+    $global->add_placeholder(
+        name        => 'text',
+        type        => 'text',
+        maxlength   => 3,
+    );
+    $global->add_placeholder(
+        name        => 'tbin',
+        type        => 'text',
+        maxlength   => 3,
+        charset     => 'binary',
+    );
+    $global->add_placeholder(
+        name        => 'bin',
+        type        => 'blob',
+        maxlength   => 3,
+    );
+    use bytes;
+    foreach my $code (0..31,128..255) {
+        my $char=chr($code).chr($code).chr($code);
+        foreach my $fname (qw(text tbin bin)) {
+            $global->put($fname => $char);
+            my $got=$global->get($fname);
+            ### dprint "char='$char', got='$got'";
+            $self->assert($char eq $got,
+                          "Bin.transparency failure on code $code, got '$got' for field '$fname' (".ord($got).")");
+        }
+    }
+}
+
 sub test_delete_field {
     my $self=shift;
 
@@ -155,7 +216,8 @@ sub test_values {
     $self->assert($cust, 'Hash object fetch failed');
 
     $cust->add_placeholder(name => 'xxx',
-                           type => 'text'
+                           type => 'text',
+                           maxlength => 20,
                           );
 
     $cust->put(name => 'foo');
@@ -248,6 +310,57 @@ sub test_integer {
     }
 }
 
+sub test_decimal {
+    my $self=shift;
+    my $odb=$self->get_odb();
+    my $cust=$odb->fetch('/Customers/c1');
+
+    $cust->add_placeholder(
+        name        => 'decimal',
+        type        => 'real',
+        minvalue    => -100,
+        scale       => 2,
+        index       => 1,
+    );
+
+    my @tests=(
+        {   value   => 0,
+            expect  => 0.00,
+        },
+        {   value   => 0.1,
+            expect  => 0.1,
+        },
+        {   value   => 0.01,
+            expect  => 0.01,
+        },
+        {   value   => 0.001,
+            expect  => 0.00,
+        },
+        {   value   => -99.999,
+            expect  => -100.00,
+        },
+        {   value   => 1234567890.01,
+            expect  => 1234567890.01,
+        },
+    );
+
+    foreach my $test (@tests) {
+        $cust->put(decimal => $test->{'value'});
+
+        my $got=$cust->get('decimal');
+
+        my $expect=$test->{'expect'};
+
+        if(abs($expect)<0.0001) {
+            $self->assert(($got ? undef : 'bad'),
+                "Expected zero value ($expect) to be a logical false, got '$got'");
+        }
+
+        $self->assert(abs($got - $expect)<0.00001,
+            "For value '$test->{'value'}' expected '$expect', got '$got'");
+    }
+}
+
 sub test_real {
     my $self=shift;
     my $odb=$self->get_odb();
@@ -317,6 +430,13 @@ sub test_real {
     my $got=$nc->get('real');
     $self->assert($got == 123.45,
                   "Got wrong real value ($got!=123.45)");
+
+    $nc->put(real => 0.000);
+
+    $got=$nc->get('real');
+
+    $self->assert(($got ? undef : 'bad'),
+        "Expected zero value to be a logical false, got '$got'");
 }
 
 sub test_unique {
@@ -326,13 +446,16 @@ sub test_unique {
     my $list=$odb->fetch('/Customers');
     $list->destroy();
 
-    foreach my $type (qw(text integer real)) {
+    foreach my $type (qw(text blob integer real)) {
 
         my $c=$list->get_new();
 
-        $c->add_placeholder(name => 'uf',
-                            type => $type,
-                            unique => 1);
+        $c->add_placeholder(
+            name        => 'uf',
+            type        => $type,
+            unique      => 1,
+            maxlength   => ($type eq 'text' || $type eq 'blob') ? 100 : undef,
+        );
 
         $c->put(uf => 1);
 
@@ -343,14 +466,18 @@ sub test_unique {
         $self->assert($c1->get('uf') == 1,
                       "Wrong value in the unique field of the first object (1)");
         my $mistake;
-        stderr_stop();
+
+        # MySQL is noisy about mistakes that we expect. So we hide DBD
+        # messages.
+        #
+        $self->stderr_stop();
         try {
             $list->put(u2 => $c);
             $mistake=1;
         } otherwise {
             $mistake=0;
         };
-        stderr_restore();
+        $self->stderr_restore();
         $self->assert(! $mistake,
                 "Succeded in putting the same object twice, 'unique' does not work");
 
@@ -366,14 +493,14 @@ sub test_unique {
         $self->assert($c2->get('uf') == 3,
                       "Wrong value in the unique field of the first object (3)");
 
-        stderr_stop();
+        $self->stderr_stop();
         try {
             $c1->put(uf => 3);
             $mistake=1;
         } otherwise {
             $mistake=0;
         };
-        stderr_restore();
+        $self->stderr_restore();
         $self->assert(! $mistake,
                       "Succeded in storing two equal values into unique field");
         $self->assert($c1->get('uf') == 1,
@@ -384,7 +511,6 @@ sub test_unique {
     }
 }
 
-##
 # Checking how 'unique' works for second level objects. The trick with
 # them is that the field should be unique in the space of an enclosing
 # container, but two containers can have identical properties.
@@ -397,7 +523,7 @@ sub test_unique_2 {
     my $c1=$list->get('c1');
     my $c2=$list->get('c2');
 
-    foreach my $type (qw(text integer real)) {
+    foreach my $type (qw(text blob integer real)) {
         $c1->add_placeholder(
             name    => 'Orders',
             type    => 'list',
@@ -408,9 +534,10 @@ sub test_unique_2 {
         my $order=$c1->get('Orders')->get_new;
 
         $order->add_placeholder(
-            name    => 'foo',
-            type    => $type,
-            unique  => 1,
+            name        => 'foo',
+            type        => $type,
+            unique      => 1,
+            maxlength   => ($type eq 'text' || $type eq 'blob') ? 100 : undef,
         );
 
         $order->put(foo => 1);
@@ -419,7 +546,7 @@ sub test_unique_2 {
         my $c2list=$c2->get('Orders');
 
         my $mistake;
-        stderr_stop();
+        $self->stderr_stop();
         try {
             $c1list->put(o1 => $order);
             $c2list->put(o1 => $order);
@@ -428,11 +555,11 @@ sub test_unique_2 {
         otherwise {
             $mistake=1;
         };
-        stderr_restore();
+        $self->stderr_restore();
         $self->assert(! $mistake,
             "Can't put the same object into two different parents' lists");
 
-        stderr_stop();
+        $self->stderr_stop();
         try {
             $c1list->put(o2 => $order);
             $mistake=1;
@@ -440,21 +567,21 @@ sub test_unique_2 {
         otherwise {
             $mistake=0;
         };
-        stderr_restore();
+        $self->stderr_restore();
         $self->assert(! $mistake,
             "Put the same object twice (type=$type), 'unique' does not work on the second level");
 
         $order->put(foo => 2);
         $c2list->put(o2 => $order);
 
-        stderr_stop();
+        $self->stderr_stop();
         try {
             $c2list->put(o1 => $order);
             $mistake=1;
         } otherwise {
             $mistake=0;
         };
-        stderr_restore();
+        $self->stderr_restore();
         $self->assert(! $mistake,
             "Put the same object twice (type=$type), replacement");
 
@@ -468,8 +595,10 @@ sub test_unique_2 {
         $self->assert($c2list->get('o2')->get('foo') eq '2',
             "Got wrong value from c2list/o2");
 
+        dprint "Dropping placeholder Orders/foo";
         $order->drop_placeholder('foo');
 
+        dprint "Dropping placeholder Orders";
         $c1->drop_placeholder('Orders');
     }
 }
@@ -483,7 +612,8 @@ sub test_get_multi {
     $self->assert($cust, 'Hash object fetch failed');
 
     $cust->add_placeholder(name => 'xxx',
-                           type => 'text'
+                           type => 'text',
+                           maxlength => 50,
                           );
 
     $cust->put(name => 'foo', xxx => '123');
@@ -531,10 +661,17 @@ sub test_null {
 
     $cust->add_placeholder(name     => 'text',
                            type     => 'text',
+                           maxlength=> 50,
                           );
     $cust->add_placeholder(name     => 'text2',
                            type     => 'text',
                            default  => 'test',
+                           maxlength=> 50,
+                          );
+    $cust->add_placeholder(name     => 'blob',
+                           type     => 'blob',
+                           default  => "\x80\x82\x84\x86",
+                           maxlength=> 50,
                           );
     $cust->add_placeholder(name     => 'integer',
                            type     => 'integer',
@@ -584,6 +721,10 @@ sub test_null {
             name    => 'text2',
             default => 'test',
         },
+        78 => {
+            name    => 'blob',
+            default => "\x80\x82\x84\x86",
+        },
     );
 
     foreach my $test (map { $matrix{$_} } sort keys %matrix) {
